@@ -2,7 +2,7 @@
 ## unit tests will not be done if RUnit is not available
 
 # Unit tests are currently failing due to scoping issues
-if(FALSE & require("RUnit", quietly=TRUE)) {
+if(FALSE && require("RUnit", quietly=TRUE)) {
  
   ## --- Setup ---
  
